#include "envy.h"
#include "envy_config.h"
#include "envy_engine.h"

static Envy_Config *_envy_config_new(void);
static Envy_Config *_envy_config_load(Envy_Config *ec, const char config_path[PATH_MAX]);

Envy_Config *
envy_config_get(void)
{
  Envy_Config *ec;
  const char *xdg_config;
  char config_path[PATH_MAX];

  xdg_config = getenv("XDG_CONFIG_HOME");
  if (xdg_config != NULL)
    {
      strcpy(config_path, xdg_config);
      strcat(config_path, "/envy.cfg");
    }
  else
    {
      const char *home;

      home = getenv("HOME");
      strcpy(config_path, home);
      strcat(config_path, "/.config");
      strcat(config_path, "/envy.cfg");
    }

  ec = _envy_config_new();
  if (!access(config_path, R_OK))
    {
      ec = _envy_config_load(ec, config_path);
    }
  else
    {
      fprintf(stderr, "Envy Config file %s isn't readable\n", config_path);
    }

  return ec;
}

static Envy_Config *
_envy_config_new(void)
{
  Envy_Config *ec;

  ec = calloc(1, sizeof(Envy_Config));
  ec->ecore_evas_engine_type = ECORE_EVAS_ENGINE_SOFTWARE_X11;
  strcpy(ec->theme, PACKAGE_DATA_DIR "/themes/strawberry.edj");
  ec->auto_osdb = 0;
  ec->langage[0] = '\0';
  ec->langage[3] = '\0';
  ec->keep_osdb_subfilename = 0;
  ec->sub_padding = 10;

  return ec;
}

static Envy_Config *
_envy_config_load(Envy_Config *ec, const char config_path[PATH_MAX])
{
  FILE *fd;

  fd = fopen(config_path, "r");

  while (!feof(fd))
    {
      char key[512], value[4096];

      if (fscanf(fd, "%512s = %4096s\n", key, value) == 2)
        {
          if (*key == '#')
            {
              continue;
            }
          else if (!strcmp(key, "theme"))
            {
              strcpy(ec->theme, value);
            }
          else if (!strcmp(key, "auto_osdb"))
            {
              ec->auto_osdb = atoi(value);
            }
          else if (!strcmp(key, "language"))
            {
              strncpy(ec->langage, value, 3);
            }
          else if (!strcmp(key, "keep_osdb_subfilename"))
            {
              ec->keep_osdb_subfilename = atoi(value);
            }
          else if (!strcmp(key, "sub_padding"))
            {
              ec->sub_padding = atoi(value);
            }
          else if (!strcmp(key, "evas_engine"))
            {
              int engine_type = envy_engine_type(value);
              if (engine_type != -1)
                ec->ecore_evas_engine_type =(Ecore_Evas_Engine_Type)engine_type;
            }
        }
    }

  return ec;
}
